@@ -4,6 +4,7 @@ from pyvariantfilter.utils import parse_csq_field, get_info_field_dict, compound
 from pysam import VariantFile
 import itertools
 import pandas as pd
+from copy import deepcopy
 
 class VariantSet:
 	"""
@@ -53,7 +54,7 @@ class VariantSet:
 			self.variant_dict[variant.variant_id] = variant
 
 
-	def read_variants_from_vcf(self, vcf_file, parse_csq=True, vep_csq_key='CSQ', proband_variants_only=True, filter_func=None, args=None):
+	def read_variants_from_vcf(self, vcf_file, parse_csq=True, vep_csq_key='CSQ', proband_variants_only=True, filter_func=None, add_gt_extras=True, args=None):
 		"""
 		Read variants from a standard VCF. Must have AD,GQ and DP fields in the Format section for each sample.
 
@@ -92,7 +93,7 @@ class VariantSet:
 
 			csq_fields = str(bcf_in.header.info[vep_csq_key].record)
 
-			csq_fields = csq_fields = csq_fields.strip()
+			csq_fields = csq_fields.strip()
 
 			index = csq_fields.index('Format:') + 8
 
@@ -158,6 +159,15 @@ class VariantSet:
 						
 						gts.append(ref_and_alt[allele])
 
+
+				if len(gts) == 1 and (chrom == 'X' or chrom == 'Y'):
+
+					if family_member_id in self.family.get_male_family_members():
+						gts.append(gts[0])
+					else:
+						gts.append(gts[0])
+						print (f'Warning: A haploid genotype at position {pos} but this sample is female.')
+
 				if gts[0] == '.' and gts[1] == '.':
 
 						ads.append(0)
@@ -200,8 +210,16 @@ class VariantSet:
 				if dp == None:
 
 					dp  = 0
+
+				if add_gt_extras:
+
+					gt_extras = dict(sample_genotype_data)
+
+				else:
+
+					gt_extras = {}
 				
-				new_variant.add_genotype(family_member_id, gts, ads, gq, dp)
+				new_variant.add_genotype(family_member_id, gts, ads, gq, dp, extras=gt_extras)
 
 			passes_filter = True
 
@@ -221,7 +239,7 @@ class VariantSet:
 
 				self.add_variant(new_variant)
 
-	def read_variants_from_platypus_vcf(self, vcf_file, parse_csq=True, vep_csq_key='CSQ', proband_variants_only=True, filter_func=None, args=None):
+	def read_variants_from_platypus_vcf(self, vcf_file, parse_csq=True, vep_csq_key='CSQ', proband_variants_only=True, filter_func=None, add_gt_extras=True, args=None):
 		"""
 		Read variants from a platypus VCF. Must have NR, NV and GQ fields in the Format section for each sample.
 
@@ -259,7 +277,7 @@ class VariantSet:
 
 			csq_fields = str(bcf_in.header.info[vep_csq_key].record)
 
-			csq_fields = csq_fields = csq_fields.strip()
+			csq_fields = csq_fields.strip()
 
 			index = csq_fields.index('Format:') + 8
 
@@ -326,6 +344,15 @@ class VariantSet:
 						
 						gts.append(ref_and_alt[allele])
 
+
+				if len(gts) == 1 and (chrom == 'X' or chrom == 'Y'):
+
+					if family_member_id in self.family.get_male_family_members():
+						gts.append(gts[0])
+					else:
+						gts.append(gts[0])
+						print (f'Warning: A haploid genotype at position {pos} but this sample is female.')
+
 				if gts[0] == '.' and gts[1] == '.':
 
 						ads.append(0)
@@ -359,8 +386,15 @@ class VariantSet:
 
 					dp  = 0
 
-				
-				new_variant.add_genotype(family_member_id, gts, ads, gq, dp)
+				if add_gt_extras:
+
+					gt_extras = dict(sample_genotype_data)
+
+				else:
+
+					gt_extras = {}
+
+				new_variant.add_genotype(family_member_id, gts, ads, gq, dp, extras=add_gt_extras)
 
 			passes_filter = True
 
@@ -455,13 +489,24 @@ class VariantSet:
 			
 					for gene in self.variant_dict[variant].get_genes(feature_key=feature_key):
 
-						if gene not in self.candidate_compound_het_dict:
+						csqs = self.variant_dict[variant].get_transcript_consequence(gene)
 
-							self.candidate_compound_het_dict[gene] = [self.variant_dict[variant]]
+						proceed = False
+						for csq in csqs:
 
-						else:
+							if csq in consequences:
+								proceed = True
+								break
 
-							self.candidate_compound_het_dict[gene].append(self.variant_dict[variant])
+						if proceed:
+
+							if gene not in self.candidate_compound_het_dict:
+
+								self.candidate_compound_het_dict[gene] = [self.variant_dict[variant]]
+
+							else:
+
+								self.candidate_compound_het_dict[gene].append(self.variant_dict[variant])
 
 
 	def get_unfiltered_compound_hets_as_dict(self):
@@ -597,10 +642,10 @@ class VariantSet:
 	def to_df(self, add_inheritance=True,
 				 lenient=False,
 				 low_penetrance_genes={},
-				 min_parental_gq_dn=30,
+				 min_parental_gq_dn=10,
 				 min_parental_depth_dn=10,
 				 max_parental_alt_ref_ratio_dn=0.04,
-				 min_parental_gq_upi=30,
+				 min_parental_gq_upi=10,
 				 min_parental_depth_upi=10):
 		"""
 		Convert variant_dict to Pandas DataFrame.
@@ -614,51 +659,51 @@ class VariantSet:
 		df_list = []
 
 		for variant in self.variant_dict :
-    
-		    var = self.variant_dict [variant]
-		    
-		    for transcript in var.transcript_annotations:
-		        
-		        row = {}
-		        
-		        row['chromosome'] = var.chrom
-		        row['position'] = var.pos
-		        row['ref'] = var.ref
-		        row['alt'] = var.alt
-		        row['filter_status'] = '|'.join(var.filter_status)
-		        row['family_id'] = var.family.family_id
-		        row['variant_id'] = var.variant_id
+	
+			var = self.variant_dict [variant]
+			
+			for transcript in var.transcript_annotations:
+				
+				row = {}
+				
+				row['chromosome'] = var.chrom
+				row['position'] = var.pos
+				row['ref'] = var.ref
+				row['alt'] = var.alt
+				row['filter_status'] = '|'.join(var.filter_status)
+				row['family_id'] = var.family.family_id
+				row['variant_id'] = var.variant_id
 
-		        if add_inheritance == True:
+				if add_inheritance == True:
 
-		        	row['inheritance_models'] = '|'.join(var.get_matching_inheritance_models(compound_het_dict= self.final_compound_hets,
-		        																			lenient=lenient,
-		        																			min_parental_gq_dn=min_parental_gq_dn,
-		        																			min_parental_depth_dn =min_parental_depth_dn,
-		        																			max_parental_alt_ref_ratio_dn= max_parental_alt_ref_ratio_dn,
-		        																			min_parental_gq_upi = min_parental_gq_upi,
-		        																			min_parental_depth_upi = min_parental_depth_upi
-		        																			))
+					row['inheritance_models'] = '|'.join(var.get_matching_inheritance_models(compound_het_dict= self.final_compound_hets,
+																							lenient=lenient,
+																							min_parental_gq_dn=min_parental_gq_dn,
+																							min_parental_depth_dn =min_parental_depth_dn,
+																							max_parental_alt_ref_ratio_dn= max_parental_alt_ref_ratio_dn,
+																							min_parental_gq_upi = min_parental_gq_upi,
+																							min_parental_depth_upi = min_parental_depth_upi
+																							))
 
-		        row['worst_consequence'] = var.get_worst_consequence()
-		        
-		        for sample in var.genotypes:
-		                   
-		            row[f'{sample}_GT'] = '/'.join(var.genotypes[sample]['genotype'])
-		            row[f'{sample}_AD'] =  ','.join(str(x) for x in var.genotypes[sample]['allele_depths'])
-		            row[f'{sample}_DP'] = var.genotypes[sample]['depth']
-		            row[f'{sample}_GQ'] = var.genotypes[sample]['genotype_quality']
-		            
-		        for info_annotation in var.info_annotations:
-		            
-		            row[f'info_{info_annotation}'] = var.info_annotations[info_annotation]
-		            
-		        for transcript_annotation in transcript:
-		            
-		            row[f'csq_{transcript_annotation}'] = transcript[transcript_annotation]
-		        
-		        
-		        df_list.append(row)
+				row['worst_consequence'] = var.get_worst_consequence()
+				
+				for sample in var.genotypes:
+						   
+					row[f'{sample}_GT'] = '/'.join(var.genotypes[sample]['genotype'])
+					row[f'{sample}_AD'] =  ','.join(str(x) for x in var.genotypes[sample]['allele_depths'])
+					row[f'{sample}_DP'] = var.genotypes[sample]['depth']
+					row[f'{sample}_GQ'] = var.genotypes[sample]['genotype_quality']
+					
+				for info_annotation in var.info_annotations:
+					
+					row[f'info_{info_annotation}'] = var.info_annotations[info_annotation]
+					
+				for transcript_annotation in transcript:
+					
+					row[f'csq_{transcript_annotation}'] = transcript[transcript_annotation]
+				
+				
+				df_list.append(row)
 
 		df = pd.DataFrame(df_list)
 
@@ -666,14 +711,297 @@ class VariantSet:
 
 
 
+	def to_dict(self, add_inheritance=True,
+				 lenient=False,
+				 low_penetrance_genes={},
+				 min_parental_gq_dn=10,
+				 min_parental_depth_dn=10,
+				 max_parental_alt_ref_ratio_dn=0.04,
+				 min_parental_gq_upi=10,
+				 min_parental_depth_upi=10,
+				 get_picked=True):
+		"""
+		Convert variant_dict to a dict fro json processing
+
+		Input: None
+
+		Returns: Python Dict object.
+
+		"""
+		variant_set_dict = {}
+
+		variant_set_dict['family'] = deepcopy(self.family.__dict__)
+
+		member_list = []
+
+		for member in variant_set_dict['family']['family_members']:
+			member_list.append(deepcopy(member.__dict__))
+
+		variant_set_dict['family']['family_members'] = deepcopy(member_list)
+
+		variant_set_dict['variants'] = []
+
+		for variant in self.variant_dict:
+
+			var = self.variant_dict[variant]
+
+			individual_variant_dict = {}
+
+			individual_variant_dict['chromosome'] = var.chrom
+			individual_variant_dict['position'] = var.pos
+			individual_variant_dict['ref'] = var.ref
+			individual_variant_dict['alt'] = var.alt
+			individual_variant_dict['filter_status'] = '|'.join(var.filter_status)
+			individual_variant_dict['end'] = var.end
+			individual_variant_dict['quality'] = var.quality
+
+			individual_variant_dict['worst_consequence'] = var.get_worst_consequence()
+
+			individual_variant_dict['all_genes'] = '|'.join(list(set(var.get_genes(feature_key='SYMBOL'))))
+
+			if add_inheritance == True:
+
+				individual_variant_dict['inheritance_models'] = '|'.join(var.get_matching_inheritance_models(compound_het_dict= self.final_compound_hets,
+																							lenient=lenient,
+																							min_parental_gq_dn=min_parental_gq_dn,
+																							min_parental_depth_dn =min_parental_depth_dn,
+																							max_parental_alt_ref_ratio_dn= max_parental_alt_ref_ratio_dn,
+																							min_parental_gq_upi = min_parental_gq_upi,
+																							min_parental_depth_upi = min_parental_depth_upi
+																							))
+
+			individual_variant_dict['info_annotations'] = deepcopy(var.info_annotations)
+
+			individual_variant_dict['transcript_annotations'] = deepcopy(var.transcript_annotations)
+
+			if get_picked == True:
+
+				individual_variant_dict['picked_transcript_annotations'] = []
+				individual_variant_dict['extra_transcript_annotations'] = []
+
+				for transcript in individual_variant_dict['transcript_annotations']:
+
+					if transcript['PICK'] == '1':
+
+						individual_variant_dict['picked_transcript_annotations'].append(transcript)
+
+					else:
+
+						individual_variant_dict['extra_transcript_annotations'].append(transcript)
 
 
+			del individual_variant_dict['transcript_annotations']
+
+			individual_variant_dict['genotypes'] = {}
+		
+			for sample in var.genotypes:
+
+					individual_variant_dict['genotypes'][sample] ={}
+					individual_variant_dict['genotypes'][sample]['genotype'] = '/'.join(var.genotypes[sample]['genotype'])
+					individual_variant_dict['genotypes'][sample]['allele_depths'] = ','.join(str(x) for x in var.genotypes[sample]['allele_depths'])
+					individual_variant_dict['genotypes'][sample]['depth'] = var.genotypes[sample]['depth']
+					individual_variant_dict['genotypes'][sample]['genotype_quality'] = var.genotypes[sample]['genotype_quality']
+					individual_variant_dict['genotypes'][sample]['extras'] = var.genotypes[sample]['extras']
+
+			variant_set_dict['variants'].append(individual_variant_dict)
+
+		return variant_set_dict
 
 
+	def read_variants_from_sv_vcf(self, vcf_file, parse_csq=True, vep_csq_key='CSQ', proband_variants_only=True, filter_func=None, args=None):
+		"""
+		Read variants from a platypus VCF. Must have NR, NV and GQ fields in the Format section for each sample.
+
+		Input:
+
+			vcf_file: (String) Path to the VCF file to read.
+			parse_csq: (Boolean) Whether to attempt to parse the CSQ field added by VEP.
+			vep_csq_key: (String) The key of the CSQ field in the VCF INFO section.
+			proband_variants_only (Boolean) Only load variants which the proband has an alt allele.
+			import_filtered (Boolean) Whether to import variants which fail the VCF Filter
+
+		Returns:
+
+			None - loads variants into self.variant_dict
+
+		"""
+		valid_chroms = {'1': None, '2': None, '3': None, '4': None,
+		 '5': None, '6': None, '7': None, '8': None, '9': None,
+		 '10': None, '11': None, '12': None, '13': None,
+		 '14': None, '15': None, '16': None, '17': None,
+		 '18': None, '19': None, '20': None, '21': None,
+		 '22': None, 'X': None, 'Y': None, 'MT': None, 'M': None}
+
+		assert self.family != None
+
+		family_member_ids = self.family.get_all_family_member_ids()
+
+		if proband_variants_only == True:
+
+			proband_id = self.family.get_proband().get_id()
+
+		bcf_in = VariantFile(vcf_file)
+
+		if parse_csq == True:
+
+			csq_fields = str(bcf_in.header.info[vep_csq_key].record)
+
+			csq_fields = csq_fields.strip()
+
+			index = csq_fields.index('Format:') + 8
+
+			csq_fields = csq_fields[index:len(csq_fields)-2].split('|')
+
+		for rec in bcf_in.fetch():
+			
+			chrom = rec.chrom
+			pos = rec.pos
+			ref = rec.ref
+			alt = rec.alts
+			filter_status = rec.filter.keys()
+			info = rec.info
+			quality = rec.qual
+			end = rec.stop
+
+			var_id = rec.id
+
+			if 'Manta' in var_id:
+
+				caller = 'manta'
+
+			elif 'DRAGEN' in var_id:
+
+				caller = 'dragencnv'
+
+			else:
+
+				caller = 'unknown'
+
+			info_dict = get_info_field_dict(info, vep_csq_key)
+
+			# extras for svs
+			try:
+				length = info_dict['SVLEN']
+			except:
+				length = 0
+				
+			if length != 0:
+			
+				if type(length) == int:
+					
+					length = abs(length)
+					
+				else:
+					
+					length = abs(max(length))
+
+			sv_type = info_dict.get('SVTYPE')
+
+			if 'chr' in chrom:
+
+				chrom = chrom.strip('chr')
+
+			# Chromosome is correct
+			if chrom not in valid_chroms:
+
+				print (f'{chrom} is not a valid chromosome. Not entered into variant set.')
+
+				continue
+			
+			assert len(alt) == 1
+
+			alt = alt[0]
+
+			if alt == '*':
+				continue
+
+			if parse_csq == True:
+
+				csq = rec.info[vep_csq_key]
+
+				transcript_annotations = parse_csq_field(csq, csq_fields)
+
+			new_variant = Variant(chrom=chrom, pos=pos, ref=ref, alt=alt, filter_status=filter_status, quality=quality, vartype=sv_type, end=end)
+			new_variant.add_family(self.family)
+			new_variant.add_transcript_annotations(transcript_annotations)
+			new_variant.add_info_annotations(info_dict)
+
+			new_variant.info_annotations['caller'] = caller
+			new_variant.info_annotations['sv_length'] = length
+
+			for family_member_id in family_member_ids:
+				
+				sample_genotype_data = rec.samples[family_member_id]
+
+				ref_and_alt = [ref, alt]
+
+				gts =[]
+
+				for allele in sample_genotype_data['GT']:
+
+					if allele == None:
+
+						gts.append('.')
+
+					else:
+
+						gts.append(ref_and_alt[allele])
+						
+
+				if len(gts) == 1 and (chrom == 'X' or chrom == 'Y'):
+
+					if family_member_id in self.family.get_male_family_members():
+						gts.append(gts[0])
+					else:
+						gts.append(gts[0])
+						print (f'Warning: A haploid genotype at position {pos} but this sample is female.')
+
+				if gts[0] == '.' and gts[1] == '.':
+					ads = [0,0]
+
+				# gq - if not present add very high value
+				try:
+					gq = sample_genotype_data['GQ']
+				except:
+					gq = 100000
+
+				if not isinstance(gq, int):
+
+					gq = 100000
+
+				# ads - need to fake for this for sv as doesn't make sense
+				if gts.count(alt) == 0:
+					
+					ads = [10000,0]
+				
+				elif gts.count(alt) == 1:
+					
+					ads = [5000,5000]
+					
+				elif gts.count(alt) ==2:
+					
+					ads = [0,10000]
+				
+				# add fake depth as well
+				dp =1000
+
+				new_variant.add_genotype(family_member_id, gts, ads, gq, dp, extras=sample_genotype_data)
+
+			passes_filter = True
+
+			if filter_func != None and args != None:
+
+				passes_filter = filter_func(new_variant, *args)
+
+				assert passes_filter == True or passes_filter == False
 
 
+			if proband_variants_only == True:
 
+				if new_variant.has_alt(proband_id) == True and passes_filter == True:
 
+					self.add_variant(new_variant)
 
+			elif passes_filter == True:
 
-
+				self.add_variant(new_variant)
\ No newline at end of file
